//
//  WZCamera
//  WowzaGoCoderSDK
//
//  Copyright 2007 – 2016, Wowza Media Systems, LLC.  All rights
//  reserved.
//
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
//

#import <AVFoundation/AVFoundation.h>
#import "WowzaConfig.h"

@interface WZCamera : NSObject

typedef NS_ENUM (NSInteger, WZCameraDirection) {
    WZCameraDirectionBack = 0,
    WZCameraDirectionFront
};

typedef NS_ENUM (NSInteger, WZCameraFocusMode) {
    WZCameraFocusModeOff = 0,
    WZCameraFocusModeAuto,
    WZCameraFocusModeContinuous,
    WZCameraFocusModeCount
};

typedef NS_ENUM (NSInteger, WZCameraExposureMode) {
    WZCameraExposureModeOff = WZCameraFocusModeCount,
    WZCameraExposureModeAuto,
    WZCameraExposureModeContinuous,
    WZCameraExposureModeMax
};

extern NSInteger const WZCameraTorch;

@property (readonly) NSUInteger cameraId;
@property (readonly) WZCameraDirection direction;
@property (readonly, getter=isActivated) BOOL activated;
@property (nonatomic, readonly, nonnull) AVCaptureDevice *iOSCaptureDevice;
@property (nonatomic, nonnull, readonly) NSArray<NSValue *> *frameSizes;
@property (nonatomic, nonnull, readonly) NSArray<WowzaConfig *> *supportedPresetConfigs;
@property (nonatomic, getter=isTorchOn) BOOL torchOn;
@property (nonatomic) WZCameraFocusMode focusMode;
@property (nonatomic) WZCameraExposureMode exposureMode;

- (nonnull instancetype) initWithCaptureDevice:(nonnull AVCaptureDevice *)captureDevice;
- (nonnull instancetype) init __attribute__((unavailable("initWithCaptureDevice must be used")));

- (BOOL) supportsWidth:(NSUInteger)width;

-(BOOL) isBack;
-(BOOL) isFront;

-(BOOL) hasCapability:(NSInteger)capabilityId;

-(void) setFocusMode:(WZCameraFocusMode)focusMode focusPoint:(CGPoint)point;
-(void) setExposureMode:(WZCameraExposureMode)exposureMode exposurePoint:(CGPoint)point;

@end
