//
//  WZCameraPreview
//  WowzaGoCoderSDK
//
//  Copyright 2007 – 2016, Wowza Media Systems, LLC.  All rights
//  reserved.
//
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
//

#import <UIKit/UIKit.h>
#import <AVFoundation/AVFoundation.h>
#import "WZCamera.h"
#import "WowzaConfig.h"
#import "WZEncoder.h"

#pragma mark WZCameraPreview Class
#pragma mark -

@interface WZCameraPreview : NSObject <WZEncoder>

typedef NS_ENUM (NSInteger, WZVideoDisplayOrientation) {
    WZVideoDisplayOrientationLandscape = 0,
    WZVideoDisplayOrientationPortrait
};

#pragma mark -
#pragma mark Public Class Methods
#pragma mark -

+ (NSInteger) numberOfDeviceCameras;
+ (nonnull NSArray<WZCamera *> *) deviceCameras;

#pragma mark -
#pragma mark Public Properties
#pragma mark -

@property (nonatomic, nullable) WZCamera* camera;
@property (nonatomic, nonnull) WowzaConfig *config;
@property (nonatomic) BOOL orientationLocked;
@property (nonatomic) CGRect viewBounds;

@property (nonatomic, readonly, nullable) NSArray<WZCamera *> *cameras;
@property (nonatomic, readonly) WZVideoDisplayOrientation deviceOrientation;
@property (nonatomic, readonly) CGSize frameSize;
@property (nonatomic, readonly, getter=isPreviewActive) BOOL previewActive;

#pragma mark -
#pragma mark Constructors
#pragma mark -

-(nonnull instancetype) init __attribute__((unavailable("init is unavailable")));
-(nonnull instancetype) initWithViewAndConfig:(nonnull UIView *)containingView config:(nonnull WowzaConfig *)aConfig;
//- (nonnull instancetype)initWithContainingView:(nonnull UIView *)containingView layerIndex:(NSUInteger)layerIndex;

#pragma mark -
#pragma mark Public Instance Methods
#pragma mark -

- (void) startPreview;
- (void) stopPreview;
#pragma mark -
- (BOOL) isSwitchCameraAvailable;
- (nonnull WZCamera *) switchCamera;

@end
