//
//  Sink.h
//  WowzaBroadcaster
//
//  Copyright 2007 – 2016, Wowza Media Systems, LLC.  All rights
//  reserved.
//
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
//

#import <AVFoundation/AVFoundation.h>
#import <WowzaGoCoderSDK/WowzaConfig.h>

 /*!
 @protocol Sink
 @discussion This is the base protocol definition for a class that can consume
 the output of an encoder.
 */

@protocol WZSink <NSObject>

@optional

- (void) startSink:(nonnull WowzaConfig *)config;
- (void) stopSink;

@end

/*!
 @protocol VideoSink
 @discussion A VideoSink consumes video encoder output.
 */
@protocol WZVideoSink <WZSink>

@optional

/*!
 Called for each frame of unencoded video produced by a video encoder.
 @param imageBuffer
 The frame's image data.
 @result
 If you wish to modify the frame before encoding, return the modified frame; otherwise return nil.
 If you return a CVImageBuffer that is different from the input buffer, the SDK will release it
 after encoding. If you do return a new CVImageBuffer, in needs to be 32 bits/pixel BGRA (kCVPixelFormatType_32BGRA),
 and it needs to be the same width and height as the input buffer.
 */
- (nullable CVImageBufferRef) videoFrameWasCaptured:(nonnull CVImageBufferRef)imageBuffer;

/*!
 Called for each frame of video that was encoded by a video encoder.
 @param data
 The frame's encoded image data.
 @param time
 The frame time, in milliseconds.
 */
- (void) videoFrameWasEncoded:(nonnull CMSampleBufferRef)data time:(uint64_t)time;


/*!
 Called when the sink bitrate is reduced due to inability to keep up with encoded frames.
 @param newBitrate
 The new (updated) bitrate.
 @param oldBitrate
 The previous bitrate.
 */
- (void) videoBitrateDidChange:(NSUInteger)newBitrate previousBitrate:(NSUInteger)previousBitrate;


@end


/*!
 @protocol AudioSink
 @discussion An AudioSink consumes audio encoder output.
 */
@protocol WZAudioSink <WZSink>

@optional

/*!
 Called for each frame of unencoded video produced by a video encoder.
 @param data
    The audio frame data.
 @param time
    The frame time, in milliseconds
 */
- (void) audioFrameWasEncoded:(nonnull void *)data size:(uint32_t)size time:(uint64_t)time;

/*!
 Called whenever the audio level value changes.
 @param level
    The current level value.
 */
- (void) audioLevelDidChange:(float)level;

@end