//
//  WowzaConfig.h
//  WowzaGoCoderSDK
//
//  Copyright 2007 – 2016, Wowza Media Systems, LLC.  All rights
//  reserved.
//
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
//

#import <AVFoundation/AVFoundation.h>
#import "WZError.h"

#pragma mark WowzaConfig Class -

typedef NS_ENUM(NSInteger, WZPreset) {
//    WZPresetLimited = 0,
    WZPresetLow,
    WZPresetMedium,
//    WZPresetHigh,
    WZPresetHD720p,
    WZPresetHD1080p,
    WZPresetUHD4k,
    WZPresetCount
};

typedef NS_ENUM(NSUInteger, WZAudioChannels) {
    WZAudioChannelsMono     = 1,
    WZAudioChannelsStereo   = 2
};

typedef NS_ENUM(NSInteger, WZCaptureOrientationBehavior) {
    WZCaptureOrientationBehaviorOff,
    WZCaptureOrientationBehaviorFollowDevice,
    WZCaptureOrientationBehaviorMaintainInitial
};

@interface WowzaConfig : NSObject <NSMutableCopying, NSCopying, NSCoding>

#pragma mark - Public Class Methods -

+ (nullable NSString *) AVCaptureSessionPresetFromPreset:(WZPreset)wzPreset;
+ (CGSize) CGSizeFromPreset:(WZPreset)preset;
+ (nullable NSString *) closestAVCaptureSessionPresetByWidth:(NSUInteger)width;
#pragma mark -
+ (nonnull NSArray *) presets;
+ (nonnull NSArray *) presetConfigs;
#pragma mark -
+ (nullable NSString *) presetDescription:(WZPreset)preset;
+ (nullable NSString *) presetFrameSizeLabel:(WZPreset)preset;

#pragma mark - Constructors -

- (nonnull instancetype) initWithPreset:(WZPreset)preset;

#pragma mark - Properties -

@property (nonatomic, assign) BOOL videoEnabled;
@property (nonatomic, assign) BOOL audioEnabled;
#pragma mark -
@property (nonatomic, assign) NSUInteger videoWidth;
@property (nonatomic, assign) NSUInteger videoHeight;
@property (nonatomic, assign) NSUInteger videoFrameRate;
@property (nonatomic, assign) NSUInteger videoKeyFrameInterval;
@property (nonatomic, assign) NSUInteger videoBitrate;
#pragma mark -
@property (nonatomic, assign) NSUInteger captureOrientationBehavior;
@property (nonatomic, assign) BOOL portraitLetterBoxed;
@property (nonatomic, assign) BOOL grayscale;
#pragma mark -
@property (nonatomic, strong, nullable) NSString *hostAddress;
@property (nonatomic, assign) NSUInteger portNumber;
@property (nonatomic, strong, nullable) NSString *applicationName;
@property (nonatomic, strong, nullable) NSString *streamName;
@property (nonatomic, strong, nullable) NSString *username;
@property (nonatomic, strong, nullable) NSString *password;
#pragma mark - Read-only Properties -
@property (nonatomic, readonly) NSUInteger audioChannels;
@property (nonatomic, readonly) NSUInteger audioSampleRate;
@property (nonatomic, readonly) NSUInteger audioBitrate;
#pragma mark -
@property (nonatomic, readonly) CGSize frameSize;
@property (nonatomic, readonly, nonnull) NSString *frameSizeLabel;

#pragma mark - Public Instance Methods -

- (void) loadPreset:(WZPreset)preset;
- (nullable NSString *) toAVCaptureSessionPreset;
- (nonnull NSString *) toClosestAVCaptureSessionPreset;
#pragma mark -
- (WZPreset) toPreset;
- (WZPreset) toClosestPreset;
#pragma mark -
- (BOOL) videoConfigEqual:(nonnull WowzaConfig *)other;
- (BOOL) isPortrait;
- (BOOL) isLandscape;
- (nullable NSError *) validateForBroadcast;

@end
