//
//  WowzaGoCoder.h
//  WowzaGoCoderSDK
//
//  Copyright 2007 – 2016, Wowza Media Systems, LLC.  All rights
//  reserved.
//
//  The above copyright notice and this permission notice shall be
//  included in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
//

#import <Foundation/Foundation.h>
#import <WowzaGoCoderSDK/WowzaConfig.h>
#import <WowzaGoCoderSDK/WZStatusCallback.h>
#import <WowzaGoCoderSDK/WZCameraPreview.h>
#import <WowzaGoCoderSDK/WZSession.h>

@interface WowzaGoCoder : NSObject <WZStatusCallback>

+ (nullable NSError *) registerLicenseKey:(nonnull NSString *)licenseKey;

// Properties
/*!
 @property status
 @discussion Reflects the current live streaming session status
 */
@property (nonatomic, nonnull, copy) WowzaConfig *config;
@property (nonatomic, nullable) UIView *cameraView;

@property (nonatomic, nonnull, readonly) WZCameraPreview *cameraPreview;
@property (nonatomic, nonnull, readonly) WZSession *session;
@property (nonatomic, nonnull, readonly) WZStatus *status;
@property (nonatomic, readonly) BOOL isStreaming;

// Instance methods
/*!
 Initialize the GoCoder SDK
 @param sdkLicenseKey
    The SDK license key.
 @result
    An instance of the top level GoCoder SDK API interface
 */
- (nullable id) initWithConfig:(nullable WowzaConfig *)config;

/*!
 Starts a live streaming session
 @param wowzaConfig
    The configuration settings for the live streaming session.
 @param statusCallback
    A callback that will be invoked with streaming session status updates and errors.
 */
- (nonnull WZSession *) startStreaming:(nullable id<WZStatusCallback>)statusCallback;
- (nonnull WZSession *) startStreamingWithConfig:(nullable id<WZStatusCallback>)statusCallback config:(nonnull WowzaConfig *)aConfig;
- (nonnull WZSession *) startStreamingWithPreset:(nullable id<WZStatusCallback>)statusCallback preset:(WZPreset)aPreset;

/*!
 End the active streaming session
 @param statusCallback
    A callback that will be invoked with streaming session status updates and errors.
 */
- (nonnull WZSession *) endStreaming:(nullable id<WZStatusCallback>)statusCallback;

@end
